--[[----------------------------------------------------------------------------
	Copyright 2008 João Liborio Cardoso
	All rights reserved
	
	VolumeControl is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License version 3 as published by the Free Software Foundation.
	VolumeControl is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License (<http://www.gnu.org/licenses/>) for more details.
---------------------------------------------------------------------------]]--

VolumeControl = LibStub('LibDataBroker-1.1'):NewDataObject('VolumeControl', {
	type = 'launcher',
	label = 'Volume',
})

local IconDirectory = 'Interface\\Addons\\VolumeControl\\Textures\\%s'
local Config = 'VolumeControl_Config'


--[[ Launcher ]]--

function VolumeControl:OnClick(button)
	EnableAddOn(Config)
	LoadAddOn(Config)
	
	if not IsAddOnLoaded(Config) then
		return
	else
		VolumeControl.OnClick(self, button)
	end
end

function VolumeControl:OnTooltipShow()
	self:AddLine('Volume Control', 1, 1, 1)
	self:AddLine('<Left Click> to set volume')
	self:AddLine('<Right Click> to toggle volume')
end

function VolumeControl:UpdateIcon()
	local volume = self:GetVolume()
	if volume > 2/3 then
		volume = 'Loud'
	elseif volume > 1/3 then
		volume = 'Medium'
	elseif volume > 0 then
		volume = 'Quiet'
	else
		volume = 'Muted'
	end
	
	self.icon = format(IconDirectory, volume)
end


--[[ Volume ]]--

function VolumeControl:SetVolume(...)
	AudioOptionsSoundPanelMasterVolume:SetValue(...) --this way updates the soundOptions panel
end

function VolumeControl:GetVolume()
	return BlizzardOptionsPanel_GetCVarSafe('Sound_MasterVolume')
end


--[[ Updating Events ]]--

hooksecurefunc('SetCVar', function(cvar)
	if cvar == 'Sound_MasterVolume' then
		VolumeControl:UpdateIcon()
	end
end)
VolumeControl:UpdateIcon()