--[[---------------------------------------------------------------------------------------------------
											Efemero-1.1
											
Description: A simple library for creating timers
Author: Copyright 2008/9 João Cardoso (Jaliborc)
License: GNU General Public License - Version 3 (<http://www.gnu.org/licenses/> for more information)
Dependencies: LibStub, EmbedHandler-1.0
--]]------------------------------------------------------------------------------------------------

local lib, old = LibStub:NewLibrary('Efemero-1.1', 2)
if not lib then
	return
end

if not old then
	LibStub('EmbedHandler-1.0'):Embed(lib)
	lib.frame = CreateFrame('Frame')
	lib.timers = {}
	lib.count = 0
elseif old == 1 then --version 1 didn't have this count
	lib.count = 0
	for parent,timers in pairs(timers) do
		for id,timer in pairs(timers) do
			lib.count = lib.count + 1
		end
	end
end

local GetTime = GetTime
local timer, times, time, delay, func
local updateFrame, timers, empty = lib.frame, lib.timers, {}
local type, pairs, unpack = type, pairs, unpack --speed up!


--[[ API ]]--

function lib:SetTimer(id, delay, times, func, ...)
	if delay and type(delay) ~= 'number' then
		error('Arg "delay" must be a number or nil value. Provided ' .. tostring(delay), 2)
	elseif times and type(times) ~= 'number' and type(times) ~= 'boolean' then
		error('Arg "times" must be a number, boolean or nil value. Provided ' .. tostring(times), 2)
	end
	
	if not lib.HasTimer(self, id) then
		lib.count = lib.count + 1
		
		if lib.count == 1 then
			updateFrame:Show()
		end
	end
	
	time = GetTime()
	timers[self] = timers[self] or {}
	timers[self][id] = {
		ending = delay and times ~= true and time + delay * (times or 1),
		tickEnd = delay and time + delay,		times = times,
		delay = delay,
		func = func,
		...,
	}
end

function lib:HasTimer(id)
	timer = timers[self]
	if timer and id then
		return timer[id] and true
	end
end

function lib:IterateTimers()
	return pairs(timers[self] or empty)
end

function lib:CancelTimer(id)
	if lib.HasTimer(self, id) then
		lib.count = lib.count - 1
		if lib.count == 0 then
			updateFrame:Hide()
		end
		
		timers[self][id] = nil
	end
end

function lib:ResetTimer(id)
	if lib.HasTimer(self, id) then
		time = GetTime()
		timer = timers[self][id]
		times, delay = timer.times, timer.delay
		
		timer.ending = delay and times ~= true and time + delay * (times or 1)
		timer.tickEnd = delay and time + delay
	end
end

function lib:FireTimer(id, ...)
	if lib.HasTimer(self, id) then
		func = timers[self][id].func or id
		if type(func) == 'function' then
			return func(...)
		elseif type(self) == 'table' and type(self[func]) == 'function' then
			return self[func](self, ...)
		end
	end
end


--[[ Update Frame ]]--

if lib.count == 0 then
	updateFrame:Hide()
end

updateFrame:SetScript('OnUpdate', function()
	time = GetTime()
	for parent,timers in pairs(timers) do
		for id,timer in pairs(timers) do
			if not timer.tickEnd or timer.tickEnd <= time then
				lib.FireTimer(parent, id, unpack(timer))
				
				if timer.ending and timer.ending <= time then
					lib.CancelTimer(parent, id)
				elseif timer.tickEnd then
					timer.tickEnd = time + timer.delay
				end
			end
		end
	end
end)


--[[ Update Embeds ]]--

if old then	lib:UpdateEmbeds() end