--[[---------------------------------------------------------------------------------------------------
											EmbedHandler-1.0
											
Description:
Author: Copyright 2008 João Cardoso (Jaliborc)
License: GNU General Public License - Version 3 (<http://www.gnu.org/licenses/> for more information)
Dependencies: LibStub
--]]------------------------------------------------------------------------------------------------

local lib, old = LibStub:NewLibrary('EmbedHandler-1.0', 3)
if not lib then
	return
else
	lib.internal = {}
	lib.empty = {}
	
	if not old then
		lib.embeds = {}
	end
end

local internal, embeds, empty, data, isFunc = lib.internal, lib.embeds, lib.empty
local type, pairs, unpack, select = type, pairs, unpack, select --speed up!


--[[ User API ]]--

function lib:Embed(target, ...)
	data = {}
	for i = 1, select('#', ...) do
		data[select(i, ...)] = true
	end
	
	internal.Embed(self, target, data)
	internal.SaveData(self, target, data)
end

function lib:SpecificEmbed(target, ...)
	data = {specific = 1, ...}
	
	internal.SpecificEmbed(self, target, data)
	internal.SaveData(self, target, data)
end

function lib:IterateEmbeds()
	return pairs(embeds[self] or empty)
end
	
function lib:UpdateEmbeds()
	for target,data in lib.IterateEmbeds(self) do --it's safer to index the lib
		if data.specific ~= 1 then
			internal.Embed(self, target, data)
		else
			internal.SpecificEmbed(self, target, data)
		end
	end
end


--[[ Internal API ]]--

function internal:Embed(target, data)
	for k,v in pairs(self) do
		isFunc = type(v) == 'function'
		if (isFunc and not data[k]) or (data[k] and not isFunc) then
			target[k] = self[k]
		end
	end
end

function internal:SpecificEmbed(target, data)
	for i,k in ipairs(data) do
		target[k] = self[k]
	end
end

function internal:SaveData(target, data)
	embeds[self] = embeds[self] or {}
	embeds[self][target] = data
end

lib:UpdateEmbeds()