--[[----------------------------------------------------------------------------
	Copyright 2008 João Liborio Cardoso
	All rights reserved
	
	VolumeControl Config is part of Volume Control
	VolumeControl Config is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License version 3 as published by the Free Software Foundation.
	VolumeControl Config is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License (<http://www.gnu.org/licenses/>) for more details.
---------------------------------------------------------------------------]]--

local TooltipBorder, TooltipBackground = TOOLTIP_DEFAULT_COLOR, TOOLTIP_DEFAULT_BACKGROUND_COLOR
local VolumeData, Slider = SoundPanelOptions.Sound_MasterVolume
LibStub('Efemero-1.1'):Embed(VolumeControl)


--[[ Launcher ]]--

function VolumeControl:OnClick(button)
	if button == 'LeftButton' then
		VolumeControl:ShowSlider(self)
	elseif button == 'RightButton' then
		VolumeControl:ToggleVolume()
	end
end

function VolumeControl:ToggleVolume()
	local volume = self:GetVolume()
	if VolumeControl_RealVolume and volume == 0 then
		self:SetVolume(VolumeControl_RealVolume)
		VolumeControl_RealVolume = nil
	else
		VolumeControl_RealVolume = volume
		self:SetVolume(0)
	end
end


--[[ Slider ]]--

function VolumeControl:ShowSlider(parent)
	Slider = Slider or self:CreateSlider()
	Slider:ClearAllPoints()
	Slider:Show()
	
	local off = self:GetIconPosition(parent) or 0
	if GetScreenHeight()/2 < select(2, parent:GetCenter()) then
		Slider:SetPoint('TOP', parent, 'BOTTOM', off, 0)
	else
		Slider:SetPoint('BOTTOM', parent, 'TOP', off, 0)
	end
end

function VolumeControl:GetIconPosition(parent)
	local children = {parent:GetRegions()}
	
	for i, child in pairs(children) do
		if type(child.GetTexture) == 'function' then
			if child:GetTexture() == self.icon then
				return child:GetCenter() - parent:GetCenter()
			end
		end
	end
end

function VolumeControl:CreateSlider()
	local sliderFrame = CreateFrame('Frame', nil, UIParent)
	sliderFrame:SetHeight(130) sliderFrame:SetWidth(30)
	sliderFrame:SetFrameStrata('FULLSCREEN_DIALOG')
	sliderFrame:EnableMouse(true)
	sliderFrame:SetAlpha(0)
	sliderFrame:Hide()
	
	sliderFrame:SetBackdrop({
		bgFile = 'Interface\\Tooltips\\UI-Tooltip-Background',
		edgeFile = 'Interface\\Tooltips\\UI-Tooltip-Border',
		insets = {left = 5, right = 5, top = 5, bottom = 5},
		tileSize = 16,
		edgeSize = 16,
		tile = true,
	})
	
	sliderFrame:SetBackdropBorderColor(TooltipBorder.r, TooltipBorder.g, TooltipBorder.b)
	sliderFrame:SetBackdropColor(TooltipBackground.r, TooltipBackground.g, TooltipBackground.b)
	
	sliderFrame:SetScript('OnShow', function()
		self:FadeInSlider()
		self:SetTimers()
	end)
	
	sliderFrame:SetScript('OnHide', function()
		self:CancelTimers()
	end)
	
	sliderFrame:SetScript('OnEnter', function()
		self:FadeInSlider()
		self:CancelTimers()
	end)
	
	sliderFrame:SetScript('OnLeave', function()
		self:SetTimers()
	end)
	
	local slider = CreateFrame('Slider', 'VolumeControlSlider', sliderFrame, 'OptionsSliderTemplate')
	slider:SetHeight(100) slider:SetWidth(11)
	slider:SetPoint('CENTER', sliderFrame)
	slider:SetOrientation('VERTICAL')
	slider:EnableMouseWheel(true)
	
	slider:SetMinMaxValues(VolumeData.minValue, VolumeData.maxValue)
	slider:SetValueStep(VolumeData.valueStep)
	slider:SetValue(1 - self:GetVolume())
	
	slider:SetScript('OnValueChanged', function(slider, value)
		self:SetVolume(1 - value)
	end)
	
	slider:SetScript('OnMouseWheel', function(slider, direction)
		slider:SetValue(slider:GetValue() + slider:GetValueStep() * -direction)
	end)
	
	slider:SetScript('OnEnter', function()
		self:FadeInSlider()
		self:CancelTimers()
	end)
	
	slider:SetScript('OnLeave', function()
		self:SetTimers()
	end)
	
	hooksecurefunc('SetCVar', function(cvar, value)
		if cvar == 'Sound_MasterVolume' then
			slider:SetValue(1 - value)
		end
	end)
	
	local name = slider:GetName()
	_G[name .. 'High']:Hide()
	_G[name .. 'Low']:Hide()
	
	local thumb = _G[name .. 'Thumb']
	thumb:SetRotation(89.5) --strange, but 90º doesn't work
	thumb:SetHeight(45)
	thumb:SetWidth(45)
	
	return sliderFrame
end

--[[ Timers ]]--

function VolumeControl:SetTimers()
	self:SetTimer('FadeOutSlider', UIDROPDOWNMENU_SHOW_TIME, nil)
	self:SetTimer('CheckMouseClicks', nil, true)
end

function VolumeControl:CancelTimers()
	self:CancelTimer('CheckMouseClicks')
	self:CancelTimer('FadeOutSlider')
end

function VolumeControl:CheckMouseClicks()
	if IsMouseButtonDown('LeftButton') or IsMouseButtonDown('RightButton') or IsMouseButtonDown('MiddleButton') then
		self:FadeOutSlider()
		self:CancelTimers()
	end
end

function VolumeControl:FadeInSlider()
	local alpha = Slider:GetAlpha()
	
	UIFrameFadeRemoveFrame(Slider)
	UIFrameFadeIn(Slider, 0.1 * (1 - alpha), alpha)
end

function VolumeControl:FadeOutSlider()
	local alpha = Slider:GetAlpha()
	
	UIFrameFadeRemoveFrame(Slider)
	UIFrameFadeOut(Slider, 0.3 * alpha, alpha)
	Slider.fadeInfo.finishedFunc = function()
		Slider:Hide()
	end
end
